package com.zeocollect;

import android.util.Log;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.module.annotations.ReactModule;
import com.zeotap.collect.Collect;
import com.zeotap.collect.CollectOptions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;

@ReactModule(name = ZeoCollectModule.NAME)
public class ZeoCollectModule extends ReactContextBaseJavaModule {
    public static final String NAME = "ZeoCollect";
    CollectOptions.Builder builder;
    Collect collect;
    ReactApplicationContext mReactContext;

    public ZeoCollectModule(ReactApplicationContext reactContext) {
        super(reactContext);
        mReactContext = reactContext;
    }

    @ReactMethod
    public void initalizeBySettingOptions(ReadableMap rawOptions) {
        builder = CollectOptions.builder(mReactContext);
        HashMap<String, Object> clientOptions = rawOptions.toHashMap();

        if (clientOptions.get("android_write_key") != null && !Objects.equals(clientOptions.get("android_write_key"), "")
                && clientOptions.get("android_write_key") instanceof String) {
            builder = builder.credential((String) clientOptions.get("android_write_key"));
        }
        if (clientOptions.get("logging") != null && clientOptions.get("logging") instanceof Boolean) {
            builder = builder.enableLogging((Boolean) clientOptions.get("logging"));
        }
        if (clientOptions.get("batch_size") != null && !Objects.equals(clientOptions.get("batch_size"), "")
                && (clientOptions.get("batch_size") instanceof Integer || clientOptions.get("batch_size") instanceof Double)) {
            if (clientOptions.get("batch_size") instanceof Double) {
                builder = builder.uploadBatchSize(((Double) clientOptions.get("batch_size")).intValue());
            } else if (clientOptions.get("batch_size") instanceof Integer) {
                builder = builder.uploadBatchSize((Integer) clientOptions.get("batch_size"));
            }
        }
        if (clientOptions.get("opt_out") != null && clientOptions.get("opt_out") instanceof Boolean) {
            builder = builder.optOut((Boolean) clientOptions.get("opt_out"));
        }
        if (clientOptions.get("service_interval") != null && !Objects.equals(clientOptions.get("service_interval"), "")
                && (clientOptions.get("service_interval") instanceof Integer || clientOptions.get("service_interval") instanceof Double)) {
            if (clientOptions.get("service_interval") instanceof Double) {
                builder = builder.serviceUploadInterval(((Double) clientOptions.get("service_interval")).intValue());
            } else if (clientOptions.get("service_interval") instanceof Integer) {
                builder = builder.serviceUploadInterval((Integer) clientOptions.get("service_interval"));
            }
        }
        if (clientOptions.get("use_consent") != null && clientOptions.get("use_consent") instanceof Boolean) {
            builder = builder.useConsent((Boolean) clientOptions.get("use_consent"));
        }
        if (clientOptions.get("max_cache_size") != null && !Objects.equals(clientOptions.get("max_cache_size"), "") &&
                (clientOptions.get("max_cache_size") instanceof Integer || clientOptions.get("max_cache_size") instanceof Double)) {
            if(clientOptions.get("max_cache_size") instanceof Double){
                builder = builder.maxCacheSize(((Double) clientOptions.get("max_cache_size")).intValue());
            } else if (clientOptions.get("max_cache_size") instanceof Integer){
                builder = builder.maxCacheSize((Integer) clientOptions.get("max_cache_size"));
            }
        }
        if (clientOptions.get("user_country") != null && !Objects.equals(clientOptions.get("user_country"), "") && clientOptions.get("user_country") instanceof String) {
            builder = builder.userCountry((String) clientOptions.get("user_country"));
        }
        if (clientOptions.get("check_for_cmp") != null && clientOptions.get("check_for_cmp") instanceof Boolean) {
            builder = builder.checkForCMP((Boolean) clientOptions.get("check_for_cmp"));
        }
        if (clientOptions.get("purposes_for_tracking") != null && clientOptions.get("purposes_for_tracking") instanceof List) {
            builder = builder.purposesForTracking(convertDoubleListToIntegerList((ArrayList<Object>) clientOptions.get("purposes_for_tracking")));
        }
        if (clientOptions.get("purposes_for_identify") != null && clientOptions.get("purposes_for_identify") instanceof List) {
            builder = builder.purposesForIdentifying(convertDoubleListToIntegerList((ArrayList<Object>) clientOptions.get("purposes_for_identify")));
        }
        if (clientOptions.get("are_identities_hashed") != null && clientOptions.get("are_identities_hashed") instanceof Boolean) {
            builder = builder.areIdentitiesHashed((Boolean) clientOptions.get("are_identities_hashed"));
            if (clientOptions.get("hash_identities") != null && clientOptions.get("hash_identities") instanceof Boolean) {
                builder = builder.hashIdentities((Boolean) clientOptions.get("hash_identities"));
            }
        }
        if (clientOptions.get("check_zeotap_vendor_consent") != null && clientOptions.get("check_zeotap_vendor_consent") instanceof Boolean) {
            builder = builder.checkZeotapVendorConsent((Boolean) clientOptions.get("check_zeotap_vendor_consent"));
        }
        CollectOptions options = builder.build();
        Collect.init(options);
        collect = Collect.getInstance();
    }

    private ArrayList<Integer> convertDoubleListToIntegerList(ArrayList<Object> list) {
        ArrayList<Integer> convertedList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            if (list.get(i) instanceof Double) {
                convertedList.add(((Double) list.get(i)).intValue());
            } else if (list.get(i) instanceof Integer) {
                convertedList.add((Integer) (list.get(i)));
            }
        }
        return convertedList;
    }

    @Override
    @NonNull
    public String getName() {
        return NAME;
    }

    @ReactMethod
    public void setEventNameProperties(String eventName) {
        if (collect != null) {
            collect.setEventProperties(eventName);
        }
    }

    @ReactMethod
    public void setEventProperties(String eventName, ReadableMap eventProp) {
        if (collect != null) {
            collect.setEventProperties(eventName, eventProp.toHashMap());
        }
    }

    @ReactMethod
    public void setInstantEventNameProperties(String eventName) {
        if (collect != null) {
            collect.setInstantEventProperties(eventName);
        }
    }

    @ReactMethod
    public void setInstantEventProperties(String eventName, ReadableMap eventProp) {
        if (collect != null) {
            collect.setInstantEventProperties(eventName, eventProp.toHashMap());
        }
    }

    @ReactMethod
    public void setUserIdentities(ReadableMap userProp) {
        if (collect != null) {
            collect.setUserIdentities(userProp.toHashMap());
        }
    }

    @ReactMethod
    public void setUserProperties(ReadableMap userProp) {
        if (collect != null) {
            collect.setUserProperties(userProp.toHashMap());
        }
    }

    @ReactMethod
    public void setPageProperties(ReadableMap properties) {
        if (collect != null) {
            collect.setPageProperties(properties.toHashMap());
        }
    }

    @ReactMethod
    public void unsetUserIdentities() {
        if(collect != null) {
            collect.unsetUserIdentities();
        }
    }

    @ReactMethod
    public void deInitiate() {
        if(collect != null) {
            collect.pauseCollection();
        }
    }

    @ReactMethod
    public void reInitialize() {
        if(collect != null) {
            collect.resumeCollection();
        }
    }

    @ReactMethod
    public void setConsent(ReadableMap consent) {
        if(collect != null) {
            collect.setConsent(consent.toHashMap());
        }
    }

    @ReactMethod
    public void addAskForConsentListener(Callback cb) {
        if(collect != null) {
            collect.listenToAskForConsent((data) -> cb.invoke());
        }
    }

    @ReactMethod
    public void getZI(Callback cb) {
        if(collect != null) {
            String zi = collect.getZI();
            cb.invoke(zi);
        }
    }

    @ReactMethod
    public void resetZI() {
        if(collect != null) {
            collect.resetZI();
        }
    }
}
