import ZeotapCollect

@objc(ZeoCollect)
class ZeoCollect: NSObject {
    
    @objc func initalizeBySettingOptions(_ rawOptions: [String: NSObject]){
        var collectOptions = CollectOption()
        if let writeKey = rawOptions["ios_write_key"] {
            print("Write key: \(writeKey)")
            if writeKey is String {
                collectOptions = collectOptions.writeKey(value: writeKey as! String)
            }
        }
        if let logging = rawOptions["logging"] {
            print("logging: \(logging)")
            if logging is Bool {
                collectOptions = collectOptions.logging(value: logging as! Bool)
            }
        }
        if let batchSize = rawOptions["batch_size"] {
            print("batch_size: \(batchSize)")
            if batchSize is NSNumber {
                collectOptions = collectOptions.batchSize(value: (batchSize as! NSNumber).intValue)
            }
        }
        if let optout = rawOptions["opt_out"] {
            print("optout: \(optout)")
            if optout is Bool {
                collectOptions = collectOptions.optout(value: optout as! Bool)
            }
        }
        if let serviceInterval = rawOptions["service_interval"] {
            print("serviceInterval: \(serviceInterval)")
            if serviceInterval is NSNumber {
                collectOptions = collectOptions.eventUploadInterval(value:(serviceInterval as! NSNumber).intValue)
            }
        }
        if let useConsent = rawOptions["use_consent"] {
            print("useConsent: \(useConsent)")
            if useConsent is Bool {
                collectOptions = collectOptions.useConsent(value: useConsent as! Bool)
            }
        }
        if let maxCacheSize = rawOptions["max_cache_size"] {
            print("maxCacheSize: \(maxCacheSize)")
            if maxCacheSize is NSNumber {
                collectOptions = collectOptions.maxCacheSize(value:(maxCacheSize as! NSNumber).intValue)
            }
        }
        if let userCountry = rawOptions["user_country"] {
            print("userCountry: \(userCountry)")
            if userCountry is String {
                collectOptions = collectOptions.userCountry(code: userCountry as! String)
            }
        }
        if let checkForCMP = rawOptions["check_for_cmp"] {
            print("checkForCMP: \(checkForCMP)")
            if checkForCMP is Bool {
                collectOptions = collectOptions.checkForCMP(value: checkForCMP as! Bool)
            }
        }
        if let purposesForTracking = rawOptions["purposes_for_tracking"] {
            print("purposesForTracking: \(purposesForTracking)")
            if purposesForTracking is NSArray {
                collectOptions = collectOptions.purposesForTracking(value: (purposesForTracking as! [Int]))
            }
        }
        if let purposesForIdentify = rawOptions["purposes_for_identify"] {
            print("purposesForIdentify: \(purposesForIdentify)")
            if purposesForIdentify is NSArray {
                collectOptions = collectOptions.abcd(value: (purposesForIdentify as! [Int]))
            }
        }
        if let areIdentitiesHashed = rawOptions["are_identities_hashed"] {
            print("areIdentitiesHashed: \(areIdentitiesHashed)")
            if areIdentitiesHashed is Bool {
                collectOptions = collectOptions.areIdentitiesHashed(value: areIdentitiesHashed as! Bool)
            }
            if let hashIdentities = rawOptions["hash_identities"] {
                print("hashIdentities: \(hashIdentities)")
                if hashIdentities is Bool {
                    collectOptions = collectOptions.hashIdentities(value: hashIdentities as! Bool)
                }
            }
        }
        if let checkZeotapVendorConsent = rawOptions["check_zeotap_vendor_consent"] {
            print("checkZeotapVendorConsent: \(checkZeotapVendorConsent)")
            if checkZeotapVendorConsent is Bool {
                collectOptions = collectOptions.checkZeotapVendorConsent(value: checkZeotapVendorConsent as! Bool)
            }
        }
        let collectOption = collectOptions.build()
        print("collectOption: \(collectOption)")
        Collect.initialize(option: collectOption)
    }
    
    @objc func setEventProperties(_ eventName: String, event eventProp: NSDictionary) {
      let prop = eventProp as! Dictionary<String,NSObject>
      if !prop.isEmpty {
        Collect.getInstance()?.setEventProperties(eventName, prop)
      }
    }
    
    @objc func setEventNameProperties(_ eventName: String) {
      Collect.getInstance()?.setEventProperties(eventName)
    }
    
    @objc func setInstantEventNameProperties(_ eventName: String) {
      Collect.getInstance()?.setInstantEventProperties(eventName)
    }
    
    @objc func setInstantEventProperties(_ eventName: String, event eventProp: [String: NSObject]) {
      Collect.getInstance()?.setInstantEventProperties(eventName, eventProp)
    }
    
    @objc func setUserProperties(_ user: [String: NSObject]) {
      Collect.getInstance()?.setUserProperties(user)
    }
    
    @objc func setPageProperties(_ page: [String: NSObject]) {
      Collect.getInstance()?.setPageProperties(page)
    }
    
    @objc func setUserIdentities(_ user: [String: NSObject]) {
      Collect.getInstance()?.setUserIdentities(user)
    }
    
    @objc func setConsent(_ consent: [String: NSObject]) {
      Collect.getInstance()?.setConsent(consent: consent)
    }
    
    @objc func addAskForConsentListener(_ cb: @escaping RCTResponseSenderBlock) {
      Collect.getInstance()?.listenToAskForConsent(action: { _data in
        cb([])
      })
    }
    
    @objc func unsetUserIdentities() {
      Collect.getInstance()?.unsetUserIdentities()
    }
    
    @objc func deInitiate() {
      Collect.getInstance()?.pauseCollection()
    }
    
    @objc func reInitialize() {
      Collect.getInstance()?.resumeCollection()
    }
    
    @objc func getZI(_ callback: @escaping RCTResponseSenderBlock) {
      let zi = Collect.getInstance()?.getZI()
      callback([zi ?? ""])
    }
    
    @objc func resetZI() {
      Collect.getInstance()?.resetZI()
    }
    
}
